import re
import urllib.parse
from io import StringIO

import lxml
import lxml.html
from django.db import migrations


def sub_variadic_url(match):
    attribute = match.group(1)
    url = match.group(2)
    if '%7B%7B' in url or '%7B%25' in url:
        url = urllib.parse.unquote(url)
    return '%s="%s"' % (attribute, url)


def forward_title_migration(apps, schema_editor):
    TextCell = apps.get_model('data', 'TextCell')
    for cell in TextCell.objects.filter(page__snapshot__isnull=True):
        if cell.title:
            # already set in previous version of 0058_textcell_title
            continue

        if not cell.text:
            continue

        parser = lxml.etree.HTMLParser()
        html_document = lxml.etree.parse(StringIO(cell.text), parser)
        root = html_document.getroot()

        if root is None:
            continue

        title_nodes = root.xpath('/html/body/h2[1]')
        if not title_nodes:
            continue
        if root.xpath('/html/body/h2[1]//preceding-sibling::*'):
            # check nothing before
            continue

        assert len(title_nodes) == 1
        title_node = title_nodes[0]
        if len(title_node) != 0 or len(title_node.attrib) != 0:
            continue

        body = title_node.getparent()
        title = title_node.text or ''
        if len(title) >= 150:
            continue
        text = title_node.tail or ''
        text += body.text or ''
        text += ''.join([lxml.html.tostring(child).decode() for child in body[1:]])
        cell.title = title
        cell.text = re.sub(r'(href|src)="(.*?)"', sub_variadic_url, text)
        cell.save()


def reverse_title_migration(apps, schema_editor):
    TextCell = apps.get_model('data', 'TextCell')
    for cell in TextCell.objects.filter(page__snapshot__isnull=True):
        if cell.title:
            cell.title = ''
            cell.text = f'<h2>{cell.title}</h2>{cell.text}'
        cell.save()


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0058_textcell_title'),
    ]

    operations = [
        migrations.RunPython(forward_title_migration, reverse_title_migration),
    ]
